# Android Secondary Display library

<!-- Table of contents -->
<details>
  <summary markdown="span">
    <b>Contents</b>
  </summary>

  [[_TOC_]]
</details>

**Date:** 2023-11-07

## General notes <a name="chapter__en_general_notes"></a>

The information in this document is provided as is and no guarantee or warranty
is given that the information is fit for any particular purpose. You may not use,
download, upload, copy, print, display, perform, reproduce, publish, license, post,
transmit, or distribute any information from this document in whole or in part
without the express written authorization of ELCOM, spoločnosť s ručením obmedzeným,
Prešov (“COMPANY”). COMPANY reserves the right to change this document at any time,
at its sole discretion, without previous notice.\
COMPANY reserves the right to seek all remedies available by law and in equity
for any violation of these Terms. Any rights not expressly granted herein
are reserved. COMPANY reserves the right to release information to the proper authorities,
as a result of a violation of these Terms, or unlawful acts, if the information
is subpoenaed and/or if we deem it necessary and/or appropriate. COMPANY hereby
disclaims all warranties and conditions with regard to this document, including
all implied warranties and conditions of fitness for a particular purpose, title,
and non-infringement. The information published in this document could include
inaccuracies, errors, or omissions, that be technical or not. Changes are periodically
added to the document. COMPANY may, but is not obligated to, make improvements
and/or changes in the content described herein at any time. Further to what precedes,
COMPANY, its managers and its employees shall in no event be held liable for any
special, direct, indirect, or consequential damages or any damages whatsoever resulting
from technical problems incurred by an equipment such as a breakdown, loss of use, data,
or profits, whether in an action of equity, contract, negligence, or other tortuous
action, arising out of or in connection with the use of this document.
Likewise, COMPANY shall in no event be held liable in case of force majeure.

## About the Document <a name="chapter__en_about_doc"></a>

### History <a name="chapter__en_history"></a>

| Date       | Description
|------------|------------------------------------------------------------------
| 2023-11-07 | Initial version
| 2023-11-13 | Added minSdk version to use library
| 2024-01-08 | Changed minSdk version from 26 to 22
| 2024-01-10 | Added note to minSdk version
| 2024-01-10 | Move minSdk version information to notes

## Introduction <a name="chapter__en_introduction"></a>

- The Android Secondary Display library is an `aar` package enabling application developers
  to directly write bitmaps to the framebuffer.

## Add library to Android Studio project <a name="chapter__en_add_library_to_android_studio_project"/>

- Choose `File -> Project Structure...` from menu
- Choose `Dependencies` and then choose `app` in the `Modules section`
- In the `Declared Dependencies` section, click the `plus button`
- Choose `JAR/AAR Dependency` from the options
- In the following dialog you need to set the following:
    - To `Step 1` enter the path to the library `aar` file
    - To `Step 2` set `implementation` option
- Press `OK` button

> :memo: **Notes:**
>   - This action will integrate the `aar` dependency into the `build.gradle` file

## Use library in application <a name="chapter__en_use_library_in_application"/>

To use library in application you need to:

- Import `SecondaryDisplay` class from library
  ```java
  import eu.elcom.asd_lib.SecondaryDisplay;
  ```

> :memo: **Notes:**
>   - Library use `minSdk` version 22
>   - Application cannot use `minSdkVersion` smaller than `minSdk` used in the library

## Create instance of the SecondaryDisplay class <a name="chapter__en_create_instance_of_the_secondarydisplay_class"/>

Before use the  `SecondaryDisplay` class it is necessary to create instance
of the class:
  ```java
  SecondaryDisplay secondaryDisplay = new SecondaryDisplay();
  ```

  > :memo: **Notes:**
  >   - Because the library can perform time-consuming operations, it is not advisable
  >     to call the operations of the SecondaryDisplay class in the User Interface task
  >     because it can stop responding, but instead it is recommended to call the methods
  >     of the SecondaryDisplay class (including the constructor) in a separate background task.

## Examples <a name="chapter__en_examples"/>

### Example 1: Clear display <a name="chapter__en_clear_display"/>

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    secondaryDisplay.clearDisplay();
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 2: Get display size <a name="chapter__en_get_display_size"/>

```java
Point size = new Point();
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    secondaryDisplay.getSize(size);
    Log.d(TAG, "Display size is: " + size.x + "x" + size.y);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 3: Get pixel format <a name="chapter__en_get_pixel_format"/>

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    int pixelFormat = secondaryDisplay.getPixelFormat();
    Log.d(TAG, "Pixel format is: " + pixelFormat);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 4: Map pixel format to Bitmap config <a name="chapter__en_map_pixel_format_to_bitmap_config"/>

Before using this example you need to perform:
  - Get pixel format (see: [Example 3: Get pixel format](#chapter__en_get_pixel_format))

```java
Bitmap.Config format;
switch (pixelFormat) {
    case PixelFormat.RGBA_8888:
        format = Bitmap.Config.ARGB_8888;
    default:
        thorw new IllegalStateException("Unexpected value: " + pixelFormat);
}
```

### Example 5: Initialization of the canvas and bitmap <a name="chapter__en_initialization_of_the_canvas_and_bitmap"/>

Before using this example you need to perform:
  - Get display resolution (see: [Example 2: Get display size](#chapter__en_get_display_size))
  - Get pixel format mapping (see: [Example 4: Map pixel format to Bitmap config](#chapter__en_map_pixel_format_to_bitmap_config))

```java
Bitmap bitmap = Bitmap.createBitmap(
                        size.x,
                        size.y,
                        format
                );
bitmap.eraseColor(Color.WHITE);

Canvas canvas = new Canvas();
canvas.setBitmap(bitmap);
```

### Example 6: Draw bitmap <a name="chapter__en_draw_bitmap"/>

Before using this example you need to perform:
  - Initialize of the canvas and bitmap (see: [Example 5: Initialization of the canvas and bitmap](#chapter__en_initialization_of_the_canvas_and_bitmap))

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    secondaryDisplay.draw(bitmap);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 7: Draw line <a name="chapter__en_draw_line"/>

Before using this example you need to perform:
  - Initialize of the canvas and bitmap (see: [Example 5: Initialization of the canvas and bitmap](#chapter__en_initialization_of_the_canvas_and_bitmap))

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    // Create style with black color and fill and stroke style
    Paint paint = new Paint();
    paint.setColor(Color.BLACK);
    paint.setStyle(Paint.Style.FILL_AND_STROKE);

    // Draw line with created style from point (1,1) to point (100,1)
    canvas.drawLine(1, 1, 100, 1, paint);
    secondaryDisplay.draw(bitmap);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 8: Draw filled circle <a name="chapter__en_draw_filled_circle"/>

Before using this example you need to perform:
  - Initialize of the canvas and bitmap (see: [Example 5: Initialization of the canvas and bitmap](#chapter__en_initialization_of_the_canvas_and_bitmap))

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    // Create style with black color and fill style
    Paint paint = new Paint();
    paint.setColor(Color.BLACK);
    paint.setStyle(Paint.Style.FILL);

    // Draw filled circle with created style from point (10,10) with radius 10
    canvas.drawCircle(10, 10, 10, paint);
    secondaryDisplay.draw(bitmap);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

### Example 9: Draw text <a name="chapter__en_draw_text"/>

Before using this example you need to perform:
  - Initialize of the canvas and bitmap (see: [Example 5: Initialization of the canvas and bitmap](#chapter__en_initialization_of_the_canvas_and_bitmap))

```java
SecondaryDisplay secondaryDisplay = null;
try {
    secondaryDisplay = new SecondaryDisplay();

    String text = "Test text";

    // Create style with black color, text size 25, default font and normal font style
    Paint paint = new Paint();
    paint.setColor(Color.BLACK);
    paint.setTextSize(25);
    paint.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.NORMAL));

    // Draw text with created style from point (1,1)
    canvas.drawText(text, 1, 1, paint);
    secondaryDisplay.draw(bitmap);
} catch (IOException e) {
    Log.e(TAG, e.getMessage(), e);
}
```

> :memo: **Notes:**
>   - Another fonts:
>       - Default bold:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.DEFAULT_BOLD, Typeface.NORMAL));
>           ```
>       - Monospace:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.MONOSPACE, Typeface.NORMAL));
>           ```
>       - Sans serif:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.SANS_SERIF, Typeface.NORMAL));
>           ```
>       - Serif:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.SERIF, Typeface.NORMAL));
>           ```
>   - Another font styles:
>       - Bold:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.BOLD));
>           ```
>       - Italic:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.ITALIC));
>           ```
>       - Bold and Italic:
>           ```java
>           paint.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.BOLD_ITALIC));
>           ```
